pub struct World<S = SoaStorage> { /* private fields */ }Expand description
Implementations
Sourceimpl<S> World<S>where
S: Storage,
impl<S> World<S>where
S: Storage,
Sourcepub fn matcher<'s, Q>(
&'s self,
) -> impl Iterator<Item = <<Q as Query<'s>>::Iter as Iterator>::Item> + 's
pub fn matcher<'s, Q>( &'s self, ) -> impl Iterator<Item = <<Q as Query<'s>>::Iter as Iterator>::Item> + 's
Sourcepub fn matcher_with_entities<'s, Q>(
&'s self,
) -> impl Iterator<Item = (Entity, <<Q as Query<'s>>::Iter as Iterator>::Item)> + 's
pub fn matcher_with_entities<'s, Q>( &'s self, ) -> impl Iterator<Item = (Entity, <<Q as Query<'s>>::Iter as Iterator>::Item)> + 's
Same as World::matcher but also returns the corresponding Entity.
ⓘ
fn update(world: &mut World) {
world
.matcher_with_entities::<All<(Write<Position>, Read<Velocity>)>>()
.for_each(|(entity, (p, v))| {
p.x += v.dx;
p.y += v.dy;
});
}Sourceimpl<S> World<S>where
S: Storage + RegisterComponent,
impl<S> World<S>where
S: Storage + RegisterComponent,
Sourcepub fn append_components<A, I>(&mut self, i: I)
pub fn append_components<A, I>(&mut self, i: I)
Sourcepub fn is_entity_valid(&self, entity: Entity) -> bool
pub fn is_entity_valid(&self, entity: Entity) -> bool
Compares the version of the entity with the version in World and returns true if they
match. Because version wraps around this is not a hard guarantee.
Sourcepub fn has_component<C: Component>(&self, e: Entity) -> bool
pub fn has_component<C: Component>(&self, e: Entity) -> bool
Returns true if the entity owns the requested component.
Sourcepub fn get_component<C: Component>(&self, e: Entity) -> Option<&C>
pub fn get_component<C: Component>(&self, e: Entity) -> Option<&C>
Retrieves a component for a specific Entity.
Sourcepub fn get_component_mut<C: Component>(&mut self, e: Entity) -> Option<&mut C>
pub fn get_component_mut<C: Component>(&mut self, e: Entity) -> Option<&mut C>
Same as World::get_component but mutable.
Sourcepub fn remove_entities<I>(&mut self, entities: I)where
I: IntoIterator<Item = Entity>,
pub fn remove_entities<I>(&mut self, entities: I)where
I: IntoIterator<Item = Entity>,
Removes the specified entities from World. Those entities are now considered invalid,
which can be checked with World::is_entity_valid.